<?php
/*
* @package		SEO Banners
* @copyright	Copyright (C) 2012-2015 Emir Sakic, http://www.sakic.net. All rights reserved.
* @license		GNU/GPL, see LICENSE.TXT
*
* This program is free software; you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the
* Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
* 
* This header must not be removed. Additional contributions/changes
* may be added to this header as long as no information is deleted.
*/

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

class plgSystemSeoBanners extends JPlugin
{
    public function onAfterInitialise() {
	
		$app = JFactory::getApplication();

		if ($app->isAdmin()) {
			return;
		}

        $config = JFactory::getConfig();

		if ($config->get('sef')!='0') {
			$router	= $app->getRouter();
			require_once ( dirname(__FILE__).'/router.php' );
			$SeoBannersRouter = new SeoBannersRouter();
			$router->attachBuildRule(array($SeoBannersRouter, 'build'));
			//echo '<pre>'; print_r($router); die;
		}
	}

    public function onAfterDispatch()
	{
		$app	= JFactory::getApplication();
		
		if ($app->isAdmin()) return;
		
		$document	= JFactory::getDocument();
		$doctype	= $document->getType();
		
		JHTML::_('behavior.framework');

		// Only render for HTML output
		if ( $doctype !== 'html' ) { return; }
		
		$base	= JURI::base(true).'/';

        $document->addScriptDeclaration("var sbPath = '".$base."';");
		$seobannersJS = JURI::root(true)."/plugins/system/seobanners/seobanners.js";
        $document->addScript($seobannersJS);
	}

	/**
	 * Converting the site URL to fit to the HTTP request
	 */
    public function onAfterRender()
	{
		$app = JFactory::getApplication();

		if ($app->getName() != 'site') {
			return true;
		}

		//Replace src links
		$base	= JURI::base(true).'/';

		if (method_exists($app, 'getBody')) {
			$buffer = $app->getBody();
		} else {
			$buffer = JResponse::getBody();
		}

		$regex  = '#<a\s+href="(('.$base.')?index.php\?option=com_banners[^"]*)"([^>]*)?>#m';
		$buffer = preg_replace_callback($regex, array('plgSystemSeoBanners', 'route'), $buffer);
        $this->checkBuffer($buffer);

		if (method_exists($app, 'setBody')) {
			$app->setBody($buffer);
		} else {
			JResponse::setBody($buffer);
		}

		return true;
	}
	
    private function checkBuffer($buffer) {
        if ($buffer === null) {
            switch (preg_last_error()) {
            case PREG_BACKTRACK_LIMIT_ERROR:
                $message = "PHP regular expression limit reached (pcre.backtrack_limit)";
                break;
            case PREG_RECURSION_LIMIT_ERROR:
                $message = "PHP regular expression limit reached (pcre.recursion_limit)";
                break;
            case PREG_BAD_UTF8_ERROR:
                $message = "Bad UTF8 passed to PCRE function";
                break;
            default:
                $message = "Unknown PCRE error calling PCRE function";
            }
            JError::raiseError(500, $message);
        }
    }
	
	/**
	 * Replaces the matched tags
	 *
	 * @param	array	An array of matches (see preg_match_all)
	 * @return	string
	 */
	protected static function route(&$matches)
	{
		$link	= $matches[0];
		$url	= $matches[1];
		
		if (preg_match('/id=([0-9]+)/', $url, $matches)) {
			$id = $matches[1];
			
			$db = JFactory::getDBO();
			$query = $db->getQuery(true);
			$query->select('clickurl');
			$query->from('#__banners');
			$query->where('id = ' . (int) $id);
			$db->setQuery($query);
			$route = $db->loadResult();
			
			// replace the link
			$link = str_replace($url, $route, $link);
			
			// remove the onclick (if opening in popup)
			$link = preg_replace('/ onclick="[^"]*"/', '', $link);
			
			// add target (if opening in parent window)
			if (!stristr($link, 'target="_blank"')) {
				$link = str_replace('>', ' target="_blank">', $link);
			}
			
			// add JS code to count clicks with AJAX
			$link = str_replace('>', ' onclick="seobanner(\''.$id.'\')">', $link);
		}
		
		return $link;
	}
}